<?php

namespace Drp\NativeEventApp\Auth;

use Drp\NativeEventApp\Contracts\MagicLinkSigner;
use Drp\NativeEventApp\Contracts\TokenAuthenticatable;
use RuntimeException;

class AuthInfoFinder implements \Drp\NativeEventApp\Contracts\AuthInfoFinder
{
    /** @var callable */
    private static $authResolver;

    /** @var callable */
    private static $currentUserResolver;

    /** @var MagicLinkSigner */
    private $linkSigner;

    public function __construct(MagicLinkSigner $linkSigner)
    {
        $this->linkSigner = $linkSigner;
    }

    public static function setAuthResolver(callable $authResolver): void
    {
        self::$authResolver = $authResolver;
    }

    public static function setCurrentUserResolver(callable $currentUserResolver): void
    {
        self::$currentUserResolver = $currentUserResolver;
    }

    public function get(string $authIdentifier, string $expiry, string $signature): TokenAuthenticatable
    {
        $this->linkSigner->verify($authIdentifier, $expiry, $signature);

        return $this->retrieve($authIdentifier);
    }

    public function retrieve(string $authIdentifier): TokenAuthenticatable
    {
        throw_if(
            self::$authResolver === null,
            new RuntimeException('Auth Info resolver has not been configured. '
            . 'See \Drp\NativeEventApp\Auth\AuthInfoFinder::setAuthResolver')
        );

        return self::$authResolver->__invoke($authIdentifier);
    }

    public function current(): TokenAuthenticatable
    {
        throw_if(
            self::$currentUserResolver === null,
            new RuntimeException('Current User resolver has not been configured. '
            . 'See \Drp\NativeEventApp\Auth\AuthInfoFinder::setAuthResolver')
        );

        return self::$currentUserResolver->__invoke();
    }
}
