<?php

namespace Drp\NativeEventApp\GraphQL\Resolvers;

use Drp\NativeEventApp\Auth\Facades\AuthInfoFinder;
use Drp\NativeEventApp\Models\Page;
use Drp\NativeEventApp\Models\Registration;
use Drp\NativeEventApp\Models\Segment;
use Drp\NativeEventApp\Modules\Module;
use Drp\NativeEventApp\Templates\Compiler;

class ModuleResolver
{
    /**
     * @var \Drp\NativeEventApp\Templates\Compiler
     */
    private $compiler;

    public function __construct(Compiler $compiler)
    {
        $this->compiler = $compiler;
    }

    public function __invoke(Page $page): string
    {
        $authenticatable = AuthInfoFinder::current();
        /** @var Registration $registration */
        $registration = Registration::forEventAndAuthenticatable($page->event, $authenticatable)->first();
        $modules = $page->modules->filter(function (Module $module) use ($registration) {
            return $registration->hasAllSegments(Segment::findMany($module->viewableBy));
        });

        return $this->compiler->for($registration)
            ->render($modules);
    }
}
