<?php

namespace Drp\NativeEventApp\Migrations;

use Exception;
use Illuminate\Database\Query\Builder;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;

/**
 * @SuppressWarnings(PHPMD)
 */
abstract class ViewMigration extends TableMigration
{
    abstract public function tableName(): string;

    public function up(): void
    {
        $select = $this->select();
        $this->validateColumns($select->columns);

        $this->down();
        DB::statement("CREATE VIEW `{$this->getTableNameWithPrefix()}` AS {$select->toSql()};");
    }

    abstract protected function select(): Builder;

    protected function validateColumns(array $columns): void
    {
        $columnNames = $this->columnNames($columns);
        throw_if(
            $columnNames !== $this->columns,
            new Exception(
                'Invalid columns in select.'
                . PHP_EOL . 'Given Columns: ' . json_encode($columnNames)
                . PHP_EOL . 'Required Columns: ' . json_encode($this->columns)
            )
        );
    }

    protected function columnNames(array $columns): array
    {
        return array_map(function ($column) {
            if (Str::contains($column, 'as')) {
                return Arr::last(explode('as ', $column));
            }
            return $column;
        }, $columns);
    }

    public function down(): void
    {
        DB::statement("DROP VIEW  IF EXISTS `{$this->getTableNameWithPrefix()}`;");
    }
}
