<?php

namespace Drp\NativeEventApp\Modules;

use Drp\NativeEventApp\Modules\Types\AgendaTimeline;
use Drp\NativeEventApp\Modules\Types\Form\Button;
use Drp\NativeEventApp\Modules\Types\Form\Form;
use Drp\NativeEventApp\Modules\Types\Form\TextArea;
use Drp\NativeEventApp\Modules\Types\Form\TextBox;
use Drp\NativeEventApp\Modules\Types\Heading;
use Drp\NativeEventApp\Modules\Types\Hero;
use Drp\NativeEventApp\Modules\Types\HTMLDisplay;
use Drp\NativeEventApp\Modules\Types\Image;
use Drp\NativeEventApp\Modules\Types\Quiz;
use Drp\NativeEventApp\Modules\Types\SimpleContainer;
use Drp\NativeEventApp\Modules\Types\Text;
use Drp\NativeEventApp\Modules\Types\Touchable;
use Illuminate\Support\Arr;

class ModuleRegistry
{
    /**
     * @var array
     */
    protected $modules = [
        'Heading' => Heading::class,
        'Text' => Text::class,
        'Hero' => Hero::class,
        'Image' => Image::class,
        'Container' => SimpleContainer::class,
        'Form' => Form::class,
        'FormTextArea' => TextArea::class,
        'FormTextBox' => TextBox::class,
        'FormButton' => Button::class,
        'AgendaTimeline' => AgendaTimeline::class,
        'Quiz' => Quiz::class,
        'HTMLDisplay' => HTMLDisplay::class,
        'MissingModule' => MissingModule::class,
        'Touchable' => Touchable::class,
    ];

    public function get(): array
    {
        return $this->modules;
    }

    public function modulesForEnums(): array
    {
        return collect($this->modules)
            ->map(function ($moduleClass, $moduleName) {
                return [
                    'name' => $moduleName,
                    'value' => $moduleName,
                    'description' => $moduleClass::getDescription(),
                ];
            })
            ->toArray();
    }

    public function getKeyByClass(string $class): string
    {
        return Arr::get(array_flip($this->modules), $class);
    }
}
