<?php

namespace Drp\NativeEventApp\Modules;

use Countable;
use Drp\NativeEventApp\Exceptions\InvalidModules;
use Drp\NativeEventApp\Facades\ModulesUnSerializer;
use Drp\NativeEventApp\Models\Registration;
use Illuminate\Contracts\Support\Jsonable;
use Illuminate\Support\Traits\ForwardsCalls;

/**
 * @mixin \Illuminate\Support\Collection
 */
class Modules implements \JsonSerializable, Jsonable, Countable
{
    use ForwardsCalls;

    /** @var \Illuminate\Support\Collection */
    private $modules;

    public function __construct(array $modules = [])
    {
        $this->modules = collect($modules);

        $this->modules->each(function ($module): void {
            throw_if($module instanceof self, InvalidModules::cantAddModules());
            throw_unless($module instanceof Module, InvalidModules::mustAddModule());
        });
    }

    public static function make(array $modules = []): self
    {
        return new static($modules);
    }

    public static function fromString(string $moduleData): self
    {
        return ModulesUnSerializer::getModulesFromString($moduleData);
    }

    public static function fromArray(array $moduleData): self
    {
        return ModulesUnSerializer::getModulesFromArray($moduleData);
    }

    public function jsonSerialize(): array
    {
        return $this->modules->values()->toArray();
    }

    public function parsedContent(Registration $registration): string
    {
        $this
            ->modules
            ->map(function (Module $module) use ($registration) {
                return $module->parseContent($registration);
            });

        return $this->toJson();
    }

    /**
     * @param int|string $options
     */
    public function toJson($options = 0): string
    {
        return json_encode($this, $options);
    }

    public function __toString(): string
    {
        return $this->toJson();
    }

    /**
     * Proxy a method call onto the collection items.
     *
     * @return mixed
     */
    public function __call(string $method, array $parameters)
    {
        return $this->forwardCallTo($this->modules, $method, $parameters);
    }

    /**
     * @inheritDoc
     */
    public function count(): int
    {
        return count($this->modules);
    }
}
