<?php

namespace Drp\NativeEventApp\Modules;

use Drp\NativeEventApp\Facades\ModuleRegistry;
use Illuminate\Support\Arr;
use Illuminate\Support\Collection;
use Illuminate\Support\Str;
use ReflectionClass;
use ReflectionProperty;

trait Serializable
{
    /**
     * @param int|string $options
     */
    public function toJson($options = 0): string
    {
        return json_encode($this, $options);
    }

    public function jsonSerialize(): array
    {
        $values['type'] = ModuleRegistry::getKeyByClass(get_class($this));

        return array_merge($values, $this->getAttributes());
    }

    public function getAttributes(): array
    {
        return $this->getPublicAttributes()
            ->map(function (ReflectionProperty $property) {
                $attributeName = $property->getName();

                return $this->getAttributeForSerialization($attributeName);
            })
            ->toArray();
    }

    private function getPublicAttributes(): Collection
    {
        $reflection = new ReflectionClass($this);
        $properties = $reflection->getProperties(\ReflectionProperty::IS_PUBLIC);

        return collect($properties)->keyBy->getName();
    }

    /**
     * @return mixed
     */
    private function getAttributeForSerialization(string $attributeName)
    {
        $serializeMethod = 'serialize' . Str::camel($attributeName);
        if (method_exists($this, $serializeMethod)) {
            return $this->{$serializeMethod}();
        }

        return $this->{$attributeName};
    }

    public function setAttributes(array $attributes): Module
    {
        $this
            ->getPublicAttributes()
            ->each(function (ReflectionProperty $property) use ($attributes): void {
                $attributeName = $property->getName();
                $value = Arr::get($attributes, $attributeName);
                $this->setAttributeForSerialization($attributeName, $value);
            });


        return $this;
    }

    /**
     * @param mixed $value
     */
    private function setAttributeForSerialization(string $attributeName, $value): void
    {
        $serializeMethod = 'set' . Str::camel($attributeName);
        if (method_exists($this, $serializeMethod)) {
            $this->$serializeMethod($value);
            return;
        }

        $this->{$attributeName} = $value;
    }
}
