<?php

namespace Drp\NativeEventApp\Modules;

use Illuminate\Contracts\Support\Jsonable;
use Illuminate\Contracts\Validation\Validator;
use Illuminate\Validation\ValidationRuleParser;

class SerializableValidator implements \JsonSerializable, Jsonable
{
    /** @var Validator */
    private $validator;

    public function __construct(Validator $validator)
    {
        $this->validator = $validator;
    }

    /**
     * @param int|string $options
     */
    public function toJson($options = 0): string
    {
        return json_encode($this, $options);
    }

    public function jsonSerialize(): array
    {
        $rules = $this->validator->getRules();

        return [
            'rules' => $rules,
            'messages' => $this->getMessages($rules),
        ];
    }

    private function getMessages(array $rules): array
    {
        return collect($rules)
            ->transform(function ($fieldRules, $attribute) {
                return $this->getMessagesForAttribute($attribute, $fieldRules);
            })
            ->toArray();
    }

    private function getMessagesForAttribute(string $attribute, array $fieldRules): array
    {
        return collect($fieldRules)
            ->mapWithKeys(function ($rule) use ($attribute) {
                [$rule, $parameters] = ValidationRuleParser::parse($rule);
                return [strtolower($rule) => $this->getFullMessage($attribute, $rule, $parameters)];
            })
            ->toArray();
    }

    private function getFullMessage(string $attribute, string $rule, array $parameters): string
    {
        $message = $this->getBaseMessage($attribute, $rule);

        return $this->validator->makeReplacements($message, $attribute, $rule, $parameters);
    }

    /**
     * Get the message for the rule because this method is protected
     *
     * @return mixed
     */
    private function getBaseMessage(string $attribute, string $rule)
    {
        return (function () use ($rule, $attribute) {
            return $this->getMessage($attribute, $rule);
        })->call($this->validator);
    }
}
