<?php

namespace Drp\NativeEventApp\Modules\Types\Actions;

use Illuminate\Contracts\Support\Arrayable;

class Action implements Arrayable
{
    const NAVIGATE_PUSH = 'NAVIGATE_PUSH';
    const MODAL_OPEN = 'MODAL_OPEN';

    /** @var string */
    private $type;

    /** @var array */
    private $payload = [];

    /**
     * @param string|int $identifier
     */
    public static function navigate($identifier): self
    {
        return new self(self::NAVIGATE_PUSH, ['id' => $identifier]);
    }

    /**
     * @param string|int $identifier
     */
    public static function openModel($identifier): self
    {
        return new self(self::MODAL_OPEN, ['id' => $identifier]);
    }

    public static function fromArray(array $action): self
    {
        return new self($action['type'], $action['payload']);
    }

    public function __construct(string $type, array $payload)
    {
        $this->type = $type;
        $this->payload = $payload;
    }

    public function type(): string
    {
        return $this->type;
    }

    public function payload(): array
    {
        return $this->payload;
    }

    public function toArray(): array
    {
        return [
            'type' => $this->type,
            'payload' => $this->payload,
        ];
    }
}
