<?php

namespace Drp\NativeEventApp\Templates\DataProviders;

use Drp\NativeEventApp\Templates\DataType;
use Illuminate\Support\Collection;
use Illuminate\Support\Str;
use SingleQuote\DocblockParser\DocblockParser;

class DataProvider
{
    public function __construct(array $attributes = [])
    {
        foreach ($attributes as $property => $value) {
            $this->$property = $value;
        }
    }

    public function getFields(): Collection
    {
        $reflection = new \ReflectionClass($this);

        return $this->getFieldsForClass($reflection);
    }

    private function getFieldsForClass(\ReflectionClass $class): Collection
    {
        $publicProperties = collect($class->getProperties(\ReflectionProperty::IS_PUBLIC));

        return $publicProperties->keyBy->name->map(function (\ReflectionProperty $property): array {
            $parsed = new DocblockParser($property);
            $type = $parsed->getTag('var');

            if (is_a($type, DataProvider::class, true)) {
                return $this->getFieldsForClass(new \ReflectionClass($type))->toArray();
            }

            return [
                'title' => $parsed->getTag('title', ucwords(Str::snake($property->getName(), ' '))),
                'type' => DataType::fromClassName($type)->value,
            ];
        });
    }
}
