<?php

namespace Drp\NativeEventApp\Test\Api\Auth;

use Drp\NativeEventApp\Auth\ContactDetail;
use Drp\NativeEventApp\Auth\ContactDetails;
use Drp\NativeEventApp\Auth\Facades\AuthInfoFinder;
use Drp\NativeEventApp\Enums\AuthMethod;
use Drp\NativeEventApp\Test\Api\GraphQLTestCase;
use Drp\NativeEventApp\Test\Support\Fakes\Registration;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Spatie\Snapshots\MatchesSnapshots;

class AuthInfoTest extends GraphQLTestCase
{
    use DatabaseMigrations, MatchesSnapshots;

    /** @test */
    public function get_auth_information_using_auth_identifier(): void
    {
        AuthInfoFinder::shouldReceive('get')->andReturn(new Registration);

        $query = '{
        authInfo(authIdentifier:"1", expiry:"1234", signature: "1234")
          {
            title,
            minimumAppVersion,
            contactMethods{
              method
              preview
            }
          }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function it_only_returns_the_sms_method_if_no_email_set(): void
    {
        AuthInfoFinder::shouldReceive('get')->andReturn(new class extends Registration {
            public function contactDetails(): ContactDetails
            {
                return new ContactDetails([
                    new ContactDetail(AuthMethod::SMS(), '07474747474'),
                ]);
            }
        });

        $query = '{
         authInfo(authIdentifier:"1", expiry:"1234", signature: "1234")
          {
            contactMethods{
              method
              preview
            }
          }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function it_only_returns_the_email_method_if_no_phone_number_set(): void
    {
        AuthInfoFinder::shouldReceive('get')->andReturn(new class extends Registration {
            public function contactDetails(): ContactDetails
            {
                return new ContactDetails([
                    new ContactDetail(AuthMethod::EMAIL(), 'test@example.com'),
                ]);
            }
        });

        $query = '{
         authInfo(authIdentifier:"1", expiry:"1234", signature: "1234")
          {
            contactMethods{
              method
              preview
            }
          }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }

    /** @test */
    public function it_throws_an_exception_if_no_auth_methods_are_available(): void
    {
        AuthInfoFinder::shouldReceive('get')->andReturn(new class extends Registration {
            public function contactDetails(): ContactDetails
            {
                return new ContactDetails([]);
            }
        })->once();

        $query = '{
         authInfo(authIdentifier:"1", expiry:"1234", signature: "1234")
          {
            contactMethods{
              method
              preview
            }
          }
        }';

        $this->assertMatchesJsonSnapshot($this->graphQlRequest($query));
    }
}
