<?php

namespace Drp\NativeEventApp\Test\Api;

use Drp\NativeEventApp\Models\Menu;
use Drp\NativeEventApp\Models\NavigatableItem;
use Drp\NativeEventApp\Models\Navigation;
use Drp\NativeEventApp\Models\Page;
use Spatie\Snapshots\MatchesSnapshots;

class MenuTest extends GraphQLTestCase
{
    use MatchesSnapshots;

    /** @test */
    public function can_get_menu(): void
    {
        $this->registerFakeEvent();

        $page = factory(Page::class)->create();
        $primary = NavigatableItem::create([
            'icon' => 'house',
            'text' => 'Home',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);
        $menu = Menu::create();
        $menu->items()->save($primary);

        Navigation::create([
            'primaryId' => $primary->id,
            'secondaryId' => $primary->id,
            'menuId' => $menu->id,
            'eventId' => $this->event->id,
        ]);

        $query = '{
            events(id:1){
                id,
                navigation {
                    primary {
                        icon
                        text
                    }
                    secondary {
                        icon
                        text
                    }
                    menu {
                        items {
                            icon
                            text
                            destination {
                                 __typename
                                ... on Page {
                                    id
                                }
                                ... on Link {
                                    href
                                }
                            }
                        }
                    }
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function orders_menu_items(): void
    {
        $this->registerFakeEvent();

        $page = factory(Page::class)->create();
        $primary = NavigatableItem::create([
            'icon' => 'house',
            'text' => 'Home',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);
        $secondary = NavigatableItem::create([
            'icon' => 'info',
            'text' => 'Information',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);
        $menu = Menu::create();
        $menu->items()->attach($primary, ['order' => 2]);
        $menu->items()->attach($secondary, ['order' => 1]);

        Navigation::create([
            'primaryId' => $primary->id,
            'secondaryId' => $secondary->id,
            'menuId' => $menu->id,
            'eventId' => $this->event->id,
        ]);

        $query = '{
            events(id:1){
                id,
                navigation {
                    menu {
                        items {
                            icon
                            text
                            destination {
                                 __typename
                                ... on Page {
                                    id
                                }
                                ... on Link {
                                    href
                                }
                            }
                        }
                    }
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }
}
