<?php

namespace Drp\NativeEventApp\Test\Support\Builders;

use Drp\NativeEventApp\Models\QuizAnswer;
use Drp\NativeEventApp\Models\QuizQuestion;

class QuestionBuilder
{
    /** @var array|int */
    protected $answers;

    /**
     * @var \Drp\NativeEventApp\Test\Support\Builders\AnswerBuilder
     */
    private $answerBuilder;

    public function __construct(AnswerBuilder $answerBuilder)
    {
        $this->answerBuilder = $answerBuilder;
    }

    public function makeMany(array $questions): array
    {
        return array_map(function ($question) {
            return factory(QuizQuestion::class)->make(array_merge(['quizId' => null], $question));
        }, $questions);
    }

    public function make(int $count = 1)
    {
        return factory(QuizQuestion::class, $count)->make(['quizId' => null]);
    }

    /**
     * @param int|array $answers
     */
    public function withAnswers($answers = 2): self
    {
        if (is_array($answers)) {
            $this->answers = $this->answerBuilder->makeMany($answers);

            return $this;
        }

        if (is_int($answers)) {
            $this->answers = factory(QuizAnswer::class, $answers)->make();
        }

        return $this;
    }

    public function saveAnswers(QuizQuestion $question)
    {
        /** @var QuizAnswer $answer */
        foreach ($this->answers as $answer) {
            $answer = clone $answer;
            $answer->quizQuestion()->associate($question);
            $answer->save();
        }
    }
}
