<?php

namespace Drp\NativeEventApp\Test;

use Drp\NativeEventApp\AddErrorCodes;
use Drp\NativeEventApp\ServiceProvider;

use Drp\NativeEventApp\Test\Support\InteractsWithModules;
use Drp\NativeEventApp\Test\Support\Kernel;
use Drp\NativeEventApp\Test\Support\Models\Registration;
use Illuminate\Contracts\Http\Kernel as KernelContract;
use Illuminate\Foundation\Testing\DatabaseMigrations;
use Illuminate\Notifications\NexmoChannelServiceProvider;
use Laravel\Passport\Passport;
use Laravel\Passport\PassportServiceProvider;
use Nexmo\Laravel\NexmoServiceProvider;
use Nuwave\Lighthouse\LighthouseServiceProvider;
use Orchestra\Testbench\TestCase as Orchestra;

class TestCase extends Orchestra
{
    use DatabaseMigrations;

    protected function setUp(): void
    {
        Passport::loadKeysFrom(__DIR__ . '/Support/keys');

        parent::setUp();

        $this->loadMigrationsFrom(realpath(__DIR__ . '/Support/migrations'));
        $this->withFactories(realpath(__DIR__ . '/Support/factories'));

        config()->set('native-event-app.auth.model', Registration::class);
        config()->set('auth.providers.users.model', Registration::class);
        config()->set('auth.guards.api.driver', 'passport');
        config()->set('nexmo.api_key', 'KEY');
        config()->set('nexmo.api_secret', 'SECRET');
        config()->set('services.nexmo.sms_from', 'NEA');
        config()->set('lighthouse.schema.register', __DIR__ . '/../src/GraphQL/schema/schema.graphql');
        config()->set('lighthouse.error_handlers', [AddErrorCodes::class]);
        config()->set('lighthouse.namespaces', [
            'models' => 'Drp\\NativeEventApp\\Models',
            'queries' => 'Drp\\NativeEventApp\\GraphQL\\Queries',
            'mutations' => 'Drp\\NativeEventApp\\GraphQL\\Mutations',
            'subscriptions' => 'Drp\\NativeEventApp\\GraphQL\\Subscriptions',
            'interfaces' => 'Drp\\NativeEventApp\\GraphQL\\Interfaces',
            'unions' => 'Drp\\NativeEventApp\\GraphQL\\Unions',
            'scalars' => 'Drp\\NativeEventApp\\GraphQL\\Scalars',
            'directives' => 'Drp\\NativeEventApp\\GraphQL\\Directives',
        ]);
    }

    protected function resolveApplicationHttpKernel($app)
    {
        $app->singleton(KernelContract::class, Kernel::class);
    }

    /**
     * Boot the testing helper traits.
     *
     * @return void
     */
    protected function setUpTraits()
    {
        $uses = parent::setUpTraits();

        if (isset($uses[InteractsWithModules::class])) {
            $this->registerFakeModules();
        }
    }

    protected function getPackageProviders($app)
    {
        return [
            PassportServiceProvider::class,
            LighthouseServiceProvider::class,
            NexmoChannelServiceProvider::class,
            NexmoServiceProvider::class,
            ServiceProvider::class,
        ];
    }

    protected function withoutGraphQLExceptionHandling(): void
    {
        config()->set('native-event-app.debug.disable_error_codes', true);
    }
}
