<?php


namespace Drp\NativeEventApp\Test\Unit;

use Drp\NativeEventApp\Auth\AuthInfoFinder;
use Drp\NativeEventApp\Test\Support\Fakes\MagicLinkSigner;
use Drp\NativeEventApp\Test\Support\Fakes\Registration;
use Drp\NativeEventApp\Test\TestCase;


class AuthInfoFinderTest extends TestCase
{
    public function setUp(): void
    {
        $reflection = new \ReflectionClass(AuthInfoFinder::class);
        tap($reflection->getProperty('authResolver'))->setAccessible(true)->setValue(null);
        tap($reflection->getProperty('currentUserResolver'))->setAccessible(true)->setValue(null);
    }

    /**
     * @test
     */
    public function throws_exception_if_no_auth_resolver_is_configured()
    {
        $this->expectException(\RuntimeException::class);
        $authInfoFinder = new AuthInfoFinder(new MagicLinkSigner);
        $authInfoFinder->get(1, '1234', 'SIGNED');
    }

    /**
     * @test
     */
    public function throws_exception_if_no_current_user_resolver_is_configured()
    {
        $this->expectException(\RuntimeException::class);
        $authInfoFinder = new AuthInfoFinder(new MagicLinkSigner);
        $authInfoFinder->current();
    }

    /**
     * @test
     */
    public function returns_auth_user_using_resolver()
    {
        $registration = new Registration;
        $fakeSigner = new MagicLinkSigner;
        $authInfoFinder = new AuthInfoFinder($fakeSigner);
        $authInfoFinder->setAuthResolver(function () use ($registration) {
            return $registration;
        });

        $user = $authInfoFinder->get(1, '1234', 'SIGNED');
        $this->assertEquals($registration, $user);
        $fakeSigner->assertVerified('1234', 'SIGNED');
    }

    /**
     * @test
     */
    public function returns_auth_user_without_signing()
    {
        $registration = new Registration;
        $authInfoFinder = new AuthInfoFinder(new MagicLinkSigner);
        $authInfoFinder->setAuthResolver(function () use ($registration) {
            return $registration;
        });

        $user = $authInfoFinder->retrieve(1);
        $this->assertEquals($registration, $user);
    }

    /**
     * @test
     */
    public function returns_current_user_using_resolver()
    {
        $registration = new Registration;
        $fakeSigner = new MagicLinkSigner;
        $authInfoFinder = new AuthInfoFinder($fakeSigner);
        $authInfoFinder->setCurrentUserResolver(function () use ($registration) {
            return $registration;
        });

        $this->assertEquals($registration, $authInfoFinder->current());
    }
}
