<?php

namespace Drp\NativeEventApp\Test\Unit\Modules;

use Drp\NativeEventApp\Modules\Modules;
use Drp\NativeEventApp\Modules\Types\Form\Form;
use Drp\NativeEventApp\Modules\Types\Form\TextBox;
use Drp\NativeEventApp\Modules\Types\Text;
use Drp\NativeEventApp\Test\TestCase;
use Illuminate\Support\Facades\Validator;

class FormTest extends TestCase
{
    /**
     * @test
     */
    public function form_can_be_serialized()
    {
        $validator = Validator::make([], ['input' => 'required']);

        $form = Form::make('form_1')
            ->validator($validator)
            ->children(Modules::make([
                TextBox::make('input')
                    ->label('Answer me this')
                    ->placeholder('Enter input')
                    ->value('Default value'),
                Text::make('My test'),
            ]));

        $this->assertEquals(json_encode([
            'type' => 'Form',
            'identifier' => 'form_1',
            'validator' => [
                'rules' => [
                    'input' => ['required'],
                ],
                'messages' => [
                    'input' => [
                        'required' => 'The input field is required.',
                    ],
                ],
            ],
            'children' => [
                [
                    'type' => 'FormTextBox',
                    'label' => 'Answer me this',
                    'value' => 'Default value',
                    'name' => 'input',
                    'styles' => [],
                    'className' => [],
                    'viewableBy' => [],
                    'placeholder' => 'Enter input',
                ],
                [
                    'type' => 'Text',
                    'text' => 'My test',
                    'styles' => [],
                    'className' => [],
                    'viewableBy' => [],
                ],
            ],
            'styles' => [],
            'className' => [],
            'viewableBy' => [],
        ]), $form->toJson());
    }
}
