<?php

namespace Drp\NativeEventApp\Test\Unit\Modules;

use Drp\NativeEventApp\Modules\SerializableValidator;
use Drp\NativeEventApp\Test\TestCase;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;

class SerializableValidatorTest extends TestCase
{
    /**
     * @test
     */
    public function returns_a_serialized_validator()
    {
        $validator = Validator::make([],
            [
                'field_1' => 'required|min:10',
                'field_2' => ['required', 'min:10'],
                'field_3' => [Rule::notIn(1, 2, 3)],
            ],
            [
                'field_2.required' => 'Custom message for :attribute',
                'field_2.min' => 'Enter a :name bigger than :min',
                'field_3.notIn' => 'Not in this',
            ]
        );

        $serializer = new SerializableValidator($validator);

        $this->assertEquals(
            json_encode([
                'rules' => [
                    'field_1' => ['required', 'min:10'],
                    'field_2' => ['required', 'min:10'],
                    'field_3' => ['not_in:"1","2","3"'],
                ],
                'messages' => [
                    'field_1' => ['required' => 'The field 1 field is required.', 'min' => 'The field 1 must be at least 10 characters.'],
                    'field_2' => ['required' => 'Custom message for field 2', 'min' => 'Enter a :name bigger than 10'],
                    'field_3' => ['notin' => 'The selected field 3 is invalid.'],
                ]
            ]),
            $serializer->toJson()
        );
    }
}