<?php

namespace Drp\NativeEventApp\Test\Unit;

use Drp\NativeEventApp\Enums\AuthMethod;
use Drp\NativeEventApp\Notifications\SendAuthCode;
use Drp\NativeEventApp\Test\Support\Fakes\Registration;
use Drp\NativeEventApp\Test\TestCase;
use Illuminate\Notifications\ChannelManager;
use Illuminate\Notifications\Channels\NexmoSmsChannel;
use Mockery;

class SendAuthCodeTest extends TestCase
{
    /** @test */
    public function can_send_custom_sms_message()
    {
        $channel = Mockery::spy(NexmoSmsChannel::class);
        app(ChannelManager::class)->extend('nexmo', function () use ($channel) {
            return $channel;
        });

        config(['native-event-app.verification.sms.message' => 'This is my message :code']);

        $notification = new SendAuthCode(AuthMethod::SMS(), '123456');
        $registration = new Registration();

        $registration->notify($notification);

        $channel->shouldHaveReceived('send')->with($registration, Mockery::on(function (SendAuthCode $notification) {
            $message = $notification->toNexmo();

            $this->assertEquals('This is my message 123456', $message->content);

            return true;
        }))->once();
    }
}
