<?php

namespace Drp\NativeEventApp\Test\Unit\Templates;

use Carbon\Carbon;
use Drp\NativeEventApp\Exceptions\InvalidTemplate;
use Drp\NativeEventApp\Templates\Compiler;
use Drp\NativeEventApp\Templates\DataProviders\DataProvider;
use Drp\NativeEventApp\Templates\DataProviders\DataProviderRegistry;
use Drp\NativeEventApp\Test\TestCase;

class CompilerTest extends TestCase
{
    /** @test */
    public function can_render_template()
    {
        $user = new class(['firstName' => 'John']) extends DataProvider {
            /** @var string */
            public $firstName;
        };
        $template = 'Hello {user.firstName}';

        $registry = new DataProviderRegistry([
            'user' => function () use ($user) {
                return $user;
            },
        ]);

        $compiler = new Compiler($registry);

        $rendered = $compiler->render($template);

        $this->assertEquals('Hello John', $rendered);
    }

    /** @test */
    public function can_compile_dates()
    {
        $user = new class(['startsAt' => Carbon::parse('1st January 2020')]) extends DataProvider {
            /** @var \Carbon\Carbon */
            public $startsAt;
        };
        $template = 'Join us on {user.startsAt.format(d/m/Y)}';

        $registry = new DataProviderRegistry([
            'user' => function () use ($user) {
                return $user;
            },
        ]);

        $compiler = new Compiler($registry);

        $rendered = $compiler->render($template);

        $this->assertEquals('Join us on 01/01/2020', $rendered);
    }

    /** @test */
    public function cant_call_methods_if_not_valid()
    {
        $user = new class(['title' => 'test']) extends DataProvider {
            /** @var string */
            public $title;
        };
        $template = 'Join us on {user.title.format(d/m/Y)}';

        $registry = new DataProviderRegistry([
            'user' => function () use ($user) {
                return $user;
            },
        ]);

        $compiler = new Compiler($registry);

        try {
            $compiler->render($template);
        } catch (InvalidTemplate $exception) {
            $this->assertEquals(InvalidTemplate::BAD_FUNCTION, $exception->getCode());

            return;
        }

        $this->fail('Should of thrown an InvalidTemplate exception');
    }
}
