<?php

namespace Drp\NativeEventApp\Test\Unit\Templates;

use Drp\NativeEventApp\Templates\DataProviders\DataProvider;
use Drp\NativeEventApp\Test\Support\Fakes\DataProviders\FakeHotel;
use Drp\NativeEventApp\Test\TestCase;

class DataProviderTest extends TestCase
{
    /** @test */
    public function can_get_fields_for_data_provider()
    {
        $provider = new class(['firstName' => 'John', 'lastName' => 'Smith']) extends DataProvider {
            /** @var string */
            public $firstName;

            /**
             * @title Surname
             * @var string
             */
            public $lastName;
        };

        $this->assertEquals([
            'firstName' => [
                'title' => 'First Name',
                'type' => 'string',
            ],
            'lastName' => [
                'title' => 'Surname',
                'type' => 'string',
            ],
        ], $provider->getFields()->toArray());
    }

    /** @test */
    public function can_get_fields_for_nested_providers()
    {
        $hotel = new FakeHotel(['name' => 'Grand Budapest']);
        $user = new class(['hotel' => $hotel]) extends DataProvider {
            /** @var \Drp\NativeEventApp\Test\Support\Fakes\DataProviders\FakeHotel */
            public $hotel;
        };

        $this->assertEquals([
            'hotel' => [
                'name' => [
                    'title' => 'Name',
                    'type' => 'string',
                ],
            ],
        ], $user->getFields()->toArray());
    }

    /** @test */
    public function can_detect_different_date_types()
    {
        $user = new class() extends DataProvider {
            /** @var \Carbon\Carbon */
            public $startsAt;

            /** @var \DateTime */
            public $endsAt;
        };

        $this->assertEquals([
            'startsAt' => [
                'title' => 'Starts At',
                'type' => 'date',
            ],
            'endsAt' => [
                'title' => 'Ends At',
                'type' => 'date',
            ],
        ], $user->getFields()->toArray());
    }
}
